/*
 * glvideo.h
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef GLVIDEO_H
#define GLVIDEO_H

#include <xine.h>

#include "mgl.h"



class VideoFrame
{
public:
	VideoFrame() {
		width = height = 0;
		aspectRatio = 0.0;
		format = 0;
		texFormat = GL_TEXTURE_2D;
		numOverlays = 0;
	}

	int width, height;
	int texWidth, texHeight; // for non rectangle texture
	double aspectRatio;
	int format;
	GLenum texFormat;
	int numOverlays;
	QRectF ovlRect[XINE_VORAW_MAX_OVL];
	QSize ovlTexSize[XINE_VORAW_MAX_OVL];
};



class VideoWidget;

class GLVideo : public QObject
{
	Q_OBJECT
public:

	GLVideo( VideoWidget *parent, GLFunc *glf );
	~GLVideo();

	void initGL();
	void draw( int width, int height, bool hasVideo, bool drawOvl=true );
	bool takeVideoShot( QImage &image, int width );

	int supportedVideoFormats() { return supportedFormats; }

	QPoint videoTopLeft, videoBottomRight;
	VideoFrame videoFrame;

public slots:

	void updateFrame( const int &frame_format, const int &frame_width,
		const int &frame_height, const double &frame_aspect, void *data0, void *data1, void *data2 );
	void updateOverlays( const int &num_ovl, raw_overlay_t *overlay_array );

private:

	void updateVideoTexture( bool resetTex, unsigned char *frameBuf, const int &textureUnit,
		const GLuint &textureID, const GLenum &format, const int &frameW, const int &frameH,
		const int &texW, const int &texH, const int &nBytes, const GLint &filter );
	void initFragProg();
	void initShader();

	VideoWidget *vw;

	GLuint ovlTexture[XINE_VORAW_MAX_OVL];
	GLuint videoTexture[3];
	GLuint videoPBO;
	GLuint fpyv12, fpyuy2;
	GLhandleARB FSHandle, PHandle;

 	GLFunc *func;
 	int supportedFormats;

signals:
	void wantRepaint();
};

#endif // GLVIDEO_H
