/*
 * osdbutton.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "osdbutton.h"



OsdButton::OsdButton( OsdConfig *conf, const QString &text ) : OsdObject( conf )
{
	label = text;
	texture = 0;
	labelTexture = 0;
}



OsdButton::~OsdButton()
{
	if ( texture )
		glDeleteTextures( 1, &texture );
	if ( labelTexture )
		glDeleteTextures( 1, &labelTexture );
}



void OsdButton::mouseMove( QPoint &p )
{
	bool oldState = hasFocus;

	if ( inMe( p.x(), p.y() ) )
		hasFocus = true;
	else
		hasFocus = false;

	if ( oldState!= hasFocus )
		emit wantRepaint();
}



bool OsdButton::mouseClicked( QMouseEvent *e )
{
	if ( !inMe( e->x(), e->y() ) )
		return false;
	emit clicked();
	return true;
}



void OsdButton::setIcon( const QString &path )
{
	iconPath = path;
}



bool OsdButton::load()
{
	QImage img;
	if ( !img.load( iconPath ) )
		return false;

	if ( !texture )
		glGenTextures( 1, &texture );

	int texW, texH;
	config->func->textureFromData( texture, img.width(), img.height(), img.bits(), texW, texH );
	u = 0;
	u1 = (float)img.width()/(float)texW;
	v = 0;
	v1 = (float)img.height()/(float)texH;

	return true;
}



void OsdButton::paint()
{
	if ( !iconPath.isNull() ) {
		if ( !texture && !load() )
			goto def;
		if ( hasFocus )
			glColor4f( 1, 1, 1, 1 );
		else
			glColor4f( 1, 1, 1, 0.6 );
		config->func->drawQuadTexture( texture, rect, u, v, u1, v1 );
		return;
	}

def:
	if ( !labelTexture ) {
		glGenTextures( 1, &labelTexture );
		QImage img( rect.width(), rect.height()*2, QImage::Format_ARGB32 );
		QPainter p;
		p.begin( &img );
		memset( img.bits(), 0, img.width()*img.height()*4 );
		p.setFont( config->normalFont() );

		QLinearGradient grad( QPoint(0,0), QPoint(0,rect.height()));
		grad.setColorAt(0, config->buttonGrad0Color() );
		grad.setColorAt(1, config->buttonGrad1Color() );
		p.fillRect( 0, 0, img.width(), rect.height(), grad );
		p.setPen( config->foregroundColor() );
		p.drawText( 0,0,img.width(),rect.height(), Qt::AlignCenter, label );

		grad = QLinearGradient( QPoint(0,rect.height()), QPoint(0,img.height()));
		grad.setColorAt(0, config->buttonHighGrad0Color() );
		grad.setColorAt(1, config->buttonHighGrad1Color() );
		p.fillRect( 0, rect.height(), img.width(), rect.height(), grad );
		p.setPen( config->foregroundHighColor() );
		p.drawText( 0,rect.height(),img.width(),rect.height(), Qt::AlignCenter, label );

		p.end();

		int texW, texH;
		config->func->textureFromData( labelTexture, img.width(), img.height(), img.bits(), texW, texH );
		lu = 0;
		lu1 = (float)img.width()/(float)texW;
		lv = 0;
		lv1 = (float)rect.height()/(float)texH;
		lv2 = (float)img.height()/(float)texH;
	}
	GLfloat nv, nv1;
	if ( hasFocus ) {
		nv = lv1;
		nv1 = lv2;
	}
	else {
		nv = lv;
		nv1 = lv1;
	}
	glColor4f( 1, 1, 1, 1 );
	config->func->drawQuadTexture( labelTexture, rect, lu, nv, lu1, nv1 );
}
